package uulm.teamname.marvelous.gamelibrary.json.basic;

import uulm.teamname.marvelous.gamelibrary.events.Event;
import uulm.teamname.marvelous.gamelibrary.json.MessageType;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Objects;

/** Represents a message sent between client and server and contains all possible data. */
public class EventMessage {

    /** This is the message type. It is either REQUESTS or EVENTS, everything else would be invalid here */
    public MessageType messageType;

    /** The list of {@link Event}s sent inside the message. */
    public Event[] messages;

    /** The type of the custom content sent with the message. */
    public String customContentType;

    /** The decoded contents of the custom content sent. */
    public HashMap<String, Object> customContent;

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        EventMessage that = (EventMessage) o;
        return messageType == that.messageType && Arrays.equals(messages, that.messages) && Objects.equals(customContentType, that.customContentType) && Objects.equals(customContent, that.customContent);
    }

    @Override
    public int hashCode() {
        int result = Objects.hash(messageType, customContentType, customContent);
        result = 31 * result + Arrays.hashCode(messages);
        return result;
    }

    @Override
    public String toString() {
        return "MessageStructure{" +
                "messageType=" + messageType +
                ", messages=" + Arrays.toString(messages) +
                ", customContentType='" + customContentType + '\'' +
                ", customContent=" + customContent +
                '}';
    }
}
